# $Id: 06_multiquery.t,v 1.2 2007/04/04 09:14:23 pythontech Exp $
use strict;
use Test::More tests => 25;

use lib 't/lib';
use DbUtil;

use PythonTech::Triples::Memory;
use PythonTech::Triples::Multi;

my $m1 = new PythonTech::Triples::Memory();
my $m2 = new PythonTech::Triples::Memory();
my $tr = new PythonTech::Triples::Multi(dbs => [$m1, $m2]);

# Create test data
$m1->add('a', 'b', 'c');
$m2->add('a', 'b', 'd');

is($tr->test('a','b','c'), 1, 'simple test');
is($tr->test('a','b','d'), 1, 'simple test');
is($tr->test('a','b','a'), undef, 'test no, resource exists');
is($tr->test('x','b','c'), undef, 'test no, subject unknown');
is($tr->test('a','y','c'), undef, 'test no, predicate unknown');
is($tr->test('a','b','z'), undef, 'test no, object unknown');
is(&qt($tr->query('a','b',undef)), 'c;d', 'query sp?');
is(&qt($tr->query('a',undef,'c')), 'b', 'query s?p');
is(&qt($tr->query('a',undef,undef)), 'b,c;b,d', 'query s??');
is(&qt($tr->query(undef,'b','c')), 'a', 'query ?po');
is(&qt($tr->query(undef,'b',undef)), 'a,c;a,d', 'query ?p?');
is(&qt($tr->query(undef,undef,'d')), 'a,b', 'query ??o');
is(&qt($tr->query(undef,undef,undef)), 'a,b,c;a,b,d', 'query ???');
is(&j($tr->sp2o('a','b')), 'c,d', 'sp2o');
is(&j($tr->sp2o('a','y')), '', 'sp2o no match');
like(scalar($tr->sp2o('a','b')), qr/^(c|d)$/, 'sp2o scalar');
is(&j($tr->po2s('b','d')), 'a', 'po2s');
is(&j(scalar $tr->po2s('b','d')), 'a', 'po2s scalar');
is(&j($tr->po2s('b','z')), '', 'po2s no match');
is(&j($tr->s2p('a')), 'b', 's2p');
is(&j($tr->s2p('c')), '', 's2p no match');

# Now with duplicate
$m2->add('a', 'b', 'c');
is(&j($tr->sp2o('a','b')), 'c,c,d', 'sp2o dups');
is(&j($tr->po2s('b','c')), 'a,a', 'po2s dups');

my $trd = new PythonTech::Triples::Multi(dbs => [$m1,$m2], nodups => 1);
is(&j($trd->sp2o('a','b')), 'c,d', 'sp2o nodups');
is(&j($trd->po2s('b','c')), 'a', 'po2s nodups');



sub qt {
    join(";", sort map {&j(@$_)} @_);
}

sub j {join(",",sort @_)}
